%{
/*
  Grammer for ypanywhere.conf

  Jeff McElroy
  jmcelroy11@sio.midco.net

*/

#include <stdio.h>
#include <string.h>
#include <pthread.h>

#include "data.h"
#include "log.h"

#include "constants.h"

extern DATA *G_data;
static DOMAIN *domain;
static MAP *map;

static char *G_mapName;
static int G_mapRefresh = DEFAULT_REFRESH;
static int G_mapMemBlock = DEFAULT_MEMBLOCK;
static int G_mapIOBlock = DEFAULT_IOBLOCK;
static char *G_mapScript;
static char *G_mapDeliminator;
static char *G_indexNames[MAX_INDEX];
static int G_indexFields[MAX_INDEX];

int G_indexCount = 0;

extern char * yytext;
extern int G_configLineNo;
extern char G_yyString[];
extern int  G_yyInteger;

void mapStart();
void mapFinish();

%}

%token START_SECTION END_SECTION END_OF_COMMAND DOMAIN_T MAP_T INDEX_T SCRIPT REFRESH MEMBLOCK_SIZE IOBLOCK_SIZE DELIMINATOR FIELD STRING INTEGER EQUALS DEBUG DEBUG_FILE

%%

config_file: /* nothing */
           | config_statement config_file
              ;

config_statement: debug
            | debug_file
            | domain
            ;

debug_file: DEBUG_FILE STRING { logSetFile(G_yyString); }
     ;

debug: DEBUG INTEGER { logSetLevel(G_yyInteger); }
     ;

domain: DOMAIN_T STRING { log(LOG_DEBUG, "adding domain %s", G_yyString); domain = dataDomainAdd(G_data, G_yyString);} '{' domain_statements '}'
      ;

domain_statements: /* nothing */
                 | domain_statement domain_statements
                 ;

domain_statement: map
                 ;

map: MAP_T STRING { mapStart(); } '{' map_statements '}' { mapFinish(); }
      ;

map_statements: /*nothing */
	      | map_statement map_statements
              ;

map_statement: SCRIPT STRING         { log(LOG_DEBUG, "map script='%s'", G_yyString);       G_mapScript = strdup(G_yyString); }
             | REFRESH INTEGER       { log(LOG_DEBUG, "map refresh=%d", G_yyInteger);       G_mapRefresh = G_yyInteger; }
             | MEMBLOCK_SIZE INTEGER { log(LOG_DEBUG, "map memblock=%d", G_yyInteger);      G_mapMemBlock= G_yyInteger; }
             | IOBLOCK_SIZE INTEGER  { log(LOG_DEBUG, "map ioblock=%d", G_yyInteger);       G_mapIOBlock = G_yyInteger; }
             | DELIMINATOR STRING    { log(LOG_DEBUG, "map deliminator='%s'", G_yyString);  G_mapDeliminator = strdup(G_yyString);}
             | index
             ;

index: INDEX_T STRING { log(LOG_DEBUG, "adding index %s", G_yyString); G_indexNames[G_indexCount] = strdup(G_yyString); } '{' index_statements '}' {G_indexCount++ ; }

      ;

index_statements: /*nothing */
	      | index_statement index_statements
              ;

index_statement: FIELD INTEGER { G_indexFields[G_indexCount] = G_yyInteger; }
             ;

%%

int yyerror(char *s)
{
  fprintf(stderr, "line %d, %s\n", G_configLineNo, s);
  return 0;
}

void mapStart()
{
  log(LOG_DEBUG, "starting map %s", G_yyString); 
  G_indexCount = 0; 
  G_mapName = strdup(G_yyString); 
}

void mapFinish()
{
  int i;
  log(LOG_DEBUG, "adding map %s", G_mapName); 
  cmdSourceMapAdd(domain, G_mapName, G_mapRefresh, G_mapScript, G_mapDeliminator, G_indexCount, G_indexNames, G_indexFields); 
  for (i=0;i<G_indexCount;i++)
    {
      free(G_indexNames[i]);
    }
  free(G_mapName);
  free(G_mapScript);
  free(G_mapDeliminator);
}



