/*
  data.h
  Jeff McElroy
  jmcelroy11@sio.midco.net
*/

/*
    ypanything, an NIS gateway to non-NIS sources
    Copyright (C) 2004  Jeff McElroy <jmcelroy11@sio.midco.net>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA                                                                 
*/

typedef struct DATA
{
  struct DOMAIN *m_domain;
}
DATA;

typedef struct DOMAIN
{
  struct DOMAIN *m_next;
  char *m_name;
  struct MAP *m_map;
}
DOMAIN;

typedef struct MAP
{
  pthread_mutex_t m_mutext;
  struct MAP *m_next;
  struct INDEX *m_index;
  char *m_name;
  void *m_data;
  void (*m_openFunc) (struct DOMAIN * domain, struct MAP * map);
  void (*m_closeFunc) (struct DOMAIN * domain, struct MAP * map);
}
MAP;

typedef struct DATUM
{
  void *m_ptr;
  int m_size;
}
DATUM;

typedef struct INDEX
{
  struct INDEX *m_next;
  char *m_name;
  void *m_data;
  
    
    DATUM (*m_getFirstFunc) (struct DOMAIN * domain, struct MAP * map,
			     struct INDEX * index, DATUM * d);
    DATUM (*m_getNextFunc) (struct DOMAIN * domain, struct MAP * map,
			    struct INDEX * index, DATUM * last);
  void (*m_closeFunc) (struct DOMAIN * domain, struct MAP * map,
		       struct INDEX * index);
}
INDEX;


DATA *dataOpen (void);
void dataClose (DATA * data);

DOMAIN *dataDomainAdd (DATA * data, char *name);
void dataDomainDelete (DATA * data, DOMAIN * domain);
DOMAIN *dataDomainFind (DATA * data, char *name);

MAP *dataMapAdd (DOMAIN * domain, char *name);
void dataMapDelete (DOMAIN * domain, MAP * map);
MAP *dataMapFind (DOMAIN * domain, char *name);
MAP *dataMapFromName (DOMAIN * domain, char *name);


INDEX *dataIndexAdd (MAP * map, char *name);
void dataIndexDelete (MAP * map, INDEX * index);
INDEX *dataIndexFind (MAP * map, char *name);
INDEX *dataIndexFromName (MAP * map, char *name);

int dataDomainMapAndIndexFromName (DATA * data, char *domainName,
				   char *mapIndexName, DOMAIN ** domainReturn,
				   MAP ** mapReturn, INDEX ** indexReturn);
void dataDump (DATA * data);

void dataMapLock (MAP * map);
void dataMapUnlock (MAP * map);
