/*
  log.c
  Jeff McElroy
  jmcelroy11@sio.midco.net
*/

/*
    ypanything, an NIS gateway to non-NIS sources
    Copyright (C) 2004  Jeff McElroy <jmcelroy11@sio.midco.net>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA                                                                 
*/

#include <errno.h>
#include <stdio.h>
#include <stdarg.h>

#include "log.h"

static int G_logLevel = LOG_ERROR;
static FILE *G_logFile = NULL;

/*void log(int logPriority, char *format, ...)*/

static char *errorLevel[] =
  { "CRITICAL", "ERROR", "MEMORY", "STATUS", "DEBUG" };

void logSetLevel (int logLevel)
{
  log (LOG_STATUS, "setting debug level %d", logLevel);

  if (logLevel < 0) {
    logLevel = 0;
  }
  else if (logLevel >= LOG_DEBUG) {
    logLevel = LOG_DEBUG;
  }

  G_logLevel = logLevel;
}

void logSetFile (char *fileName)
{
  log (LOG_DEBUG, "setting debug_file to '%s'", fileName);

  if (G_logFile != NULL) {
    fclose (G_logFile);
  }

  G_logFile = fopen (fileName, "w");

  if (G_logFile == NULL) {
    log (LOG_DEBUG, "could not open debug_file '%s', '%s'", fileName,
	 strerror (errno));
  }
}


void
log_func (char *fileName, char *function, int line, int logPriority,
	  char *format, ...)
{
  va_list ap;

  if (logPriority <= G_logLevel) {
    va_start (ap, format);
    if (G_logFile == NULL) {
      fprintf (stderr, "%s:%d:%s:%s:", fileName, line, function,
	       errorLevel[logPriority]);
      vfprintf (stderr, format, ap);
      fprintf (stderr, "\n");
      fflush (stderr);
    }
    else {
      fprintf (G_logFile, "%s:%d:%s:%s:", fileName, line, function,
	       errorLevel[logPriority]);
      vfprintf (G_logFile, format, ap);
      fprintf (G_logFile, "\n");
      fflush (G_logFile);
    }
    va_end (ap);
  }

  if (logPriority == LOG_CRIT) {
    exit (1);
  }
}
