/*
  main.c
  Jeff McElroy
  jmcelroy11@sio.midco.net
*/

/*
    ypanything, an NIS gateway to non-NIS sources
    Copyright (C) 2004  Jeff McElroy <jmcelroy11@sio.midco.net>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA                                                                 
*/

#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <pthread.h>
#include <sys/param.h>
#include <errno.h>
#include <signal.h>

#include "constants.h"
#include "heap.h"
#include "data.h"
#include "log.h"

DATA *G_data = NULL;

extern int rpcStart ();
extern yyparse ();
extern FILE *yyin;

void deamonize ()
{
  pid_t childPID;
  int fd;

  /* if started by init, then don't detach */
  if (getppid () == 1) {
    return;
  }

#if defined(SIGTTOU)
  signal (SIGTTOU, SIG_IGN);
#endif

#if defined(SIGTTIN)
  signal (SIGTTOU, SIG_IGN);
#endif

#if defined(SIGSTP)
  signal (SIGTTOU, SIG_IGN);
#endif

#if defined(SIGHUP)
  signal (SIGHUP, SIG_IGN);
#endif

  childPID = fork ();
  if (childPID < 0) {
    perror ("could not fork()");
    exit (1);
  }
  if (childPID > 0) {
    exit (0);
  }

  for (fd = 0; fd < NOFILE; fd++) {
    close (fd);
  }

  errno = 0;

}

int main (int argc, char *argv[])
{
  int c;
  int deamonFlag = 0;
  int errorFlag = 0;
  int quietFlag = 0;
  char *configFilename = DEFAULT_CONFIGFILE; 
  FILE *configFile;

  extern int optind;
  extern char *optarg;

  while ((c = getopt (argc, argv, "qdf:")) != EOF) {
    switch (c) {
    case 'd':
      deamonFlag = 1;
      break;

    case 'f':
      configFilename = optarg;
      break;

    case 'q':
      quietFlag = 1;
      break;

    default:
      errorFlag = 1;
    }
  }

  if (quietFlag == 0) {
    fprintf(stderr, "ypAnything version 1.1, Copyright (C) 2004 Jeff McElroy jmcelroy11@sio.midco.net\n");
    fprintf(stderr, "ypAnything comes with ABSOLUTELY NO WARRANTY; for details see the file\n");
    fprintf(stderr, "'COPYING'. This is free software, and you are welcome to redistribute\n");
    fprintf(stderr, "it under certain conditions; see the file 'COPYING' for details.\n\n");
  }

  if (errorFlag) {
    fprintf (stderr, "usage: ypanything [-dq] [-f configFile]\n");
    fprintf (stderr, " -d              Run program in the background as a deamon.\n");
    fprintf (stderr, " -q              Quiet mode, do not show startup copyright.\n");
    fprintf (stderr, " -f <configFile> Set the configuration file. default: ypanything.conf\n");
    return 1;
  }

  heapInitialize ();

  if (deamonFlag) {
    deamonize ();
  }

  configFile = fopen (configFilename, "r");
  if (configFile == NULL) {
    log (LOG_CRIT, "could not open the configuration file '%s': %s",
	 configFilename, strerror (errno));
    return 1;
  }

  G_data = dataOpen ();
  yyin = configFile;
  yyparse ();

  rpcStart ();

  return 0;
}
