/*
  ypanything.c
  Jeff McElroy
  jmcelroy11@sio.midco.net
*/

/*
    ypanything, an NIS gateway to non-NIS sources
    Copyright (C) 2004  Jeff McElroy <jmcelroy11@sio.midco.net>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA                                                                 
*/

#include <pthread.h>

#include "yp.h"
#include "data.h"
#include "log.h"

#include "cmdSource.h"

extern DATA *G_data;

void *ypproc_null_2_svc (void *argp, struct svc_req *rqstp)
{
  static char *result;
  static char rval = 0;
  log (LOG_DEBUG, "ypproc_null_2_svc()\n");

  result = &rval;
  return (void *) &result;
}

bool_t *ypproc_domain_2_svc (domainname * name, struct svc_req *rqstp)
{
  static bool_t result;
  struct sockaddr_in *callingHost;
  DOMAIN *domain;

  callingHost = svc_getcaller (rqstp->rq_xprt);
  log (LOG_DEBUG, "ypproc_domain_2_svc(%s)", *name);
  log (LOG_DEBUG, "ypproc_domain_2_svc() { from: %s:%d}",
       inet_ntoa (callingHost->sin_addr), ntohs (callingHost->sin_port));

  domain = dataDomainFind (G_data, *name);

  if (domain != NULL) {
    log (LOG_DEBUG, "ypproc_domain_2_svc(%s) {result = TRUE;}", *name);
    result = TRUE;
  }
  else {
    log (LOG_DEBUG, "ypproc_domain_2_svc(%s) {result = FALSE;}", *name);
    result = FALSE;
  }
  return &result;
}

bool_t *ypproc_domain_nonack_2_svc (domainname * name, struct svc_req * rqstp)
{
#if 0
  static bool_t result;

  log (LOG_DEBUG, "ypproc_domain_nonack_2_svc()");

  result = TRUE;
  return &result;
#endif

  static bool_t result;
  struct sockaddr_in *callingHost;
  DOMAIN *domain;

  callingHost = svc_getcaller (rqstp->rq_xprt);
  log (LOG_DEBUG, "ypproc_nonack_2_svc(%s)", *name);
  log (LOG_DEBUG, "ypproc_nonack_2_svc() { from: %s:%d}",
       inet_ntoa (callingHost->sin_addr), ntohs (callingHost->sin_port));

  domain = dataDomainFind (G_data, *name);

  if (domain != NULL) {
    log (LOG_DEBUG, "ypproc_nonack_2_svc(%s) {result = TRUE;}", *name);
    result = TRUE;
  }
  else {
    log (LOG_DEBUG, "ypproc_nonack_2_svc(%s) {result = FALSE;}", *name);
    result = FALSE;
  }
  return &result;

}

ypresp_val *ypproc_match_2_svc (ypreq_key * key, struct svc_req * rqstp)
{
  static ypresp_val result;
  DOMAIN *domain;
  MAP *map;
  INDEX *index;
  DATUM datumKey;
  DATUM datumResult;

  log (LOG_DEBUG, "ypproc_match_2_svc()");
  log (LOG_DEBUG, "ypproc_match_2_svc(){ key->domain=\"%s\" }", key->domain);
  log (LOG_DEBUG, "ypproc_match_2_svc(){ key->map=\"%s\" }", key->map);
  log (LOG_DEBUG, "ypproc_match_2_svc(){ key->key=\"%*.*s\" }",
       key->key.keydat_len, key->key.keydat_len, key->key.keydat_val);

  result.val.valdat_len = 0;
  result.val.valdat_val = NULL;

  if (dataDomainMapAndIndexFromName
      (G_data, key->domain, key->map, &domain, &map, &index) != 0) {
    if (domain == NULL) {
      log (LOG_DEBUG, "ypproc_match_2_svc(){result.stat = YP_NODOM;}");
      result.stat = YP_NODOM;
    }
    else {
      log (LOG_DEBUG, "ypproc_match_2_svc(){result.stat = YP_NOMAP;}");
      result.stat = YP_NOKEY;
    }
  }
  else {
    datumKey.m_ptr = key->key.keydat_val;
    datumKey.m_size = key->key.keydat_len;
    datumResult = index->m_getFirstFunc (domain, map, index, &datumKey);
    if (datumResult.m_ptr != NULL) {
      log (LOG_DEBUG, "ypproc_match_2_svc(){result.stat = YP_TRUE;}");
      result.stat = YP_TRUE;
    }
    else {
      log (LOG_DEBUG, "ypproc_match_2_svc(){result.stat = YP_NOKEY;}");
      result.stat = YP_NOKEY;
    }
    result.val.valdat_len = datumResult.m_size;
    result.val.valdat_val = datumResult.m_ptr;
    dataMapUnlock (map);
  }

  return &result;
}

ypresp_key_val *ypproc_first_2_svc (ypreq_key * key, struct svc_req * rqstp)
{
  static ypresp_key_val result;
  DOMAIN *domain;
  MAP *map;
  INDEX *index;
  DATUM datumKey;
  DATUM datumResult;

  log (LOG_DEBUG, "ypproc_first_2_svc()");

  result.val.valdat_len = 0;
  result.val.valdat_val = NULL;

  if (dataDomainMapAndIndexFromName
      (G_data, key->domain, key->map, &domain, &map, &index) != 0) {
    if (domain == NULL) {
      log (LOG_DEBUG, "ypproc_first_2_svc(){result.stat = YP_NODOM;}");
      result.stat = YP_NODOM;
    }
    else {
      log (LOG_DEBUG, "ypproc_first_2_svc(){result.stat = YP_NOMAP;}");
      result.stat = YP_NOKEY;
    }
  }
  else {
    datumKey.m_ptr = NULL;
    datumKey.m_size = 0;
    datumResult = index->m_getFirstFunc (domain, map, index, &datumKey);
    if (datumResult.m_ptr != NULL) {
      log (LOG_DEBUG, "ypproc_first_2_svc(){result.stat = YP_TRUE;}");
      result.stat = YP_TRUE;
    }
    else {
      log (LOG_DEBUG, "ypproc_first_2_svc(){result.stat = YP_NOKEY;}");
      result.stat = YP_YPERR;
    }
    result.key.keydat_len = datumKey.m_size;
    result.key.keydat_val = datumKey.m_ptr;
    result.val.valdat_len = datumResult.m_size;
    result.val.valdat_val = datumResult.m_ptr;
    dataMapUnlock (map);
  }
  return &result;
}

ypresp_key_val *ypproc_next_2_svc (ypreq_key * key, struct svc_req * rqstp)
{
  static ypresp_key_val result;
  DOMAIN *domain;
  MAP *map;
  INDEX *index;
  DATUM datumKey;
  DATUM datumResult;

  log (LOG_DEBUG, "ypproc_next_2_svc() enter");

  result.val.valdat_len = 0;
  result.val.valdat_val = NULL;

  if (dataDomainMapAndIndexFromName
      (G_data, key->domain, key->map, &domain, &map, &index) != 0) {
    if (domain == NULL) {
      log (LOG_DEBUG, "ypproc_next_2_svc(){result.stat = YP_NODOM;}");
      result.stat = YP_NODOM;
    }
    else {
      log (LOG_DEBUG, "ypproc_next_2_svc(){result.stat = YP_NOKEY;}");
      result.stat = YP_NOKEY;
    }
  }
  else {
    datumKey.m_ptr = key->key.keydat_val;
    datumKey.m_size = key->key.keydat_len;
    datumResult = index->m_getNextFunc (domain, map, index, &datumKey);
    if (datumResult.m_ptr != NULL) {
      log (LOG_DEBUG, "ypproc_next_2_svc(){result.stat = YP_TRUE;}");
      result.stat = YP_TRUE;
    }
    else {
      log (LOG_DEBUG, "ypproc_next_2_svc(){result.stat = YP_NOMORE;}");
      result.stat = YP_NOMORE;
    }
    result.key.keydat_len = datumKey.m_size;
    result.key.keydat_val = datumKey.m_ptr;
    result.val.valdat_len = datumResult.m_size;
    result.val.valdat_val = datumResult.m_ptr;
    dataMapUnlock (map);
  }

  log (LOG_DEBUG, "ypproc_next_2_svc() exit");

  return &result;
}

ypresp_xfr *ypproc_xfr_2_svc (ypreq_xfr * argp, struct svc_req * rqstp)
{
  static ypresp_xfr result;

  log (LOG_DEBUG, "ypproc_xfr_2_svc()");

  /*
   * insert server code here
   */

  return &result;
}

void *ypproc_clear_2_svc (void *argp, struct svc_req *rqstp)
{
  static char *result;
  log (LOG_DEBUG, "ypproc_clear_2_svc()");

  /*
   * insert server code here
   */

  return (void *) &result;
}

ypresp_all *ypproc_all_2_svc (ypreq_nokey * key, struct svc_req *rqstp)
{
  static ypresp_all result;
  log (LOG_DEBUG, "ypproc_all_2_svc()");

  /*
   * insert server code here
   */

  return &result;
}

ypresp_master *ypproc_master_2_svc (ypreq_nokey * argp,
				    struct svc_req * rqstp)
{
  static ypresp_master result;
  log (LOG_DEBUG, "ypproc_master_2_svc()");

  /*
   * insert server code here
   */

  return &result;
}

ypresp_order *ypproc_order_2_svc (ypreq_nokey * argp, struct svc_req * rqstp)
{
  static ypresp_order result;
  log (LOG_DEBUG, "ypproc_order_2_svc()");

  /*
   * insert server code here
   */

  return &result;
}

ypresp_maplist *ypproc_maplist_2_svc (domainname * argp,
				      struct svc_req * rqstp)
{
  static ypresp_maplist result;
  log (LOG_DEBUG, "ypproc_maplist_2_svc()");

  /*
   * insert server code here
   */

  return &result;
}

void *yppushproc_null_1_svc (void *argp, struct svc_req *rqstp)
{
  static char *result;
  log (LOG_DEBUG, "yppushproc_null_1_svc()");

  /*
   * insert server code here
   */

  return (void *) &result;
}

void *yppushproc_xfrresp_1_svc (yppushresp_xfr * argp, struct svc_req *rqstp)
{
  static char *result;
  log (LOG_DEBUG, "yppushproc_xfrresp_1_svc()");

  /*
   * insert server code here
   */

  return (void *) &result;
}
